@@ -300,8 +300,8 @@ class Sale(Workflow, ModelSQL, ModelView):
                 ).join(sale_line_invoice_line, 'LEFT',
                     condition=sale_line_invoice_line.sale_line == sale_line.id
                     ).join(move, 'LEFT',
-                        condition=(move.origin == Concat(SaleLine.__name__,
-                                sale_line.id))
+                        condition=(move.origin == Concat(
+                                SaleLine.__name__ + ',', sale_line.id))
                         ).select(sql_table.id,
                             where=(sql_table.state == 'confirmed')
                             & ((sale_line_invoice_line.id != None)
@@ -583,7 +583,11 @@ class Sale(Workflow, ModelSQL, ModelView):
         Search on shipments or returns
         '''
         def method(self, name, clause):
-            return [('lines.moves.shipment.id',) + tuple(clause[1:])
+            if isinstance(clause[2], basestring):
+                target = 'rec_name'
+            else:
+                target = 'id'
+            return [('lines.moves.shipment.' + target,) + tuple(clause[1:])
                 + (model_name,)]
         return classmethod(method)
 
@@ -1108,7 +1112,7 @@ class SaleLine(ModelSQL, ModelView):
     def get_move_done(self, name):
         Uom = Pool().get('product.uom')
         done = True
-        if not self.product:
+        if self.type != 'line' or not self.product:
             return True
         if self.product.type == 'service':
             return True
